Mon Dec  3 16:46:27 CET 2007  asf@boinkor.net
  * Implement multiple-value-or
diff -rN -u old-alexandria/control-flow.lisp new-alexandria/control-flow.lisp
--- old-alexandria/control-flow.lisp	2007-12-03 16:49:03.000000000 +0100
+++ new-alexandria/control-flow.lisp	2007-12-03 16:49:03.000000000 +0100
@@ -65,4 +65,18 @@
                               (return-from ,xor (values nil nil))
                               (setf ,true ,tmp))))
                    datums)
-         (return-from ,xor (values ,true t))))))
\ No newline at end of file
+         (return-from ,xor (values ,true t))))))
+
+(defmacro multiple-value-or (nth-value &body forms)
+  "Evaluates FORM arguments one at a time, until the NTH-VALUE returned by one
+of the forms is non-NIL. It then returns all the values returned by evaluating
+that form. If none of the forms return a non-nil nth value, this form returns
+NIL."
+  (once-only (nth-value)
+    (with-gensyms (values)
+      `(let ((,values (multiple-value-list ,(first forms))))
+         (if (nth ,nth-value ,values)
+             (values-list ,values)
+             ,(if (rest forms)
+                  `(multiple-value-or ,nth-value ,@(rest forms))
+                  nil))))))
\ No newline at end of file

